/*******************************************************************************
*
*	TITLE:		control.c 
*
*	VERSION:	0.1                           
*
*	DATE:		01-29-2008
*
*	AUTHOR:		Team 1511
*
*	DESCRIPTION:	This file contains functions for controlling the arm and manipulator.
*******************************************************************************/

#ifndef _control_h
#define _control_h

// Minimum movement of joystick required to actuate arm
#define ARM_CONTROL_DEADBAND 30

// Lowest arm POT value
#define ARM_POT_MIN 0

// Max arm POT value
#define ARM_POT_MAX (ARM_POT_MIN + 2000)

// Angle for comparisons - middle between up and down positions
#define ARM_POT_MID ((ARM_POT_MAX - ARM_POT_MIN) / 2)



// PWM delta used when driving wrist if POT is broken
#define WRIST_MANUAL_SPEED 100

// POT value to step by with button presses up and down (for PID)
#define WRIST_TARGET_STEP 10

// Lowest POT value for the wrist
#define WRIST_POT_MIN 0

// Greatest POT value for the wrist
#define WRIST_POT_MAX (WRIST_POT_MIN + 1000)

// POT value at which wrist must be at or above in order to
// stay within the 80 inch cylinder for arm raise/lower movement
#define WRIST_POT_MIN_RAISE (WRIST_POT_MIN + 100)

// Wrist angle for placing/hurdling
#define WRIST_POT_SCORE (WRIST_POT_MIN + 500)

// Wrist angle to park robot within starting box
#define WRIST_POT_PARK (WRIST_POT_MIN + 750)



// Number of 26.2ms ticks to force pump on for after manipulator
// cylinder is fired.  This should be the amount of time it takes 
// to replace the air used to actuate the manipulator cylinder.
#define MANIP_PUMP_TICKS 38

// Number of 26.2ms ticks to force pump on for after arm cylinder
// is fired.  Note that pump will stay on for quite a while after 
// this because of the pressure switch; this merely starts the 
// pump up before the pressure switch goes.
#define ARM_PUMP_TICKS 38



void Initialize_Arm(void);
void Arm_Control(void);
unsigned char Pump_Precharge(void);
int Raise_Arm(void);
int Lower_Arm(void);

#endif
