/*******************************************************************************
*
*	TITLE:		motion.h 
*
*	VERSION:	1.0                           
*
*	DATE:		01-10-2008
*
*	AUTHOR:		Team 1511
*
*	DESCRIPTION:	This file contains functions for autonomously driving the wheels.
***************************************************************************************************************************************************************/

#ifndef _motion_h
#define _motion_h

/* 2008 robot drive train encoder parameters */
/* 8-inch wheels, 21-inch wheelbase          */
/* So, 66-inch turn circumference            */
/* 128 ticks per revolution                  */

#define TICKS_PER_INCH 5L //new 2008 robot value
#define TICKS_PER_FOOT (TICKS_PER_INCH * 12L)
#define TICKS_PER_NINETY_DEGREES 140L

#define START_TO_FINISH_LINE (318 * TICKS_PER_INCH)
#define FINISH_LINE_TO_END (162 * TICKS_PER_INCH)
#define START_TO_LANE_END (366 * TICKS_PER_INCH)
#define LANE_DIV_LENGTH (324 * TICKS_PER_INCH)


// Default pwm value for forward speed while turning
#define DRIVE_FWDSPEED 150


#define default_Lane 		autoposition

#define PID_NO_OSCILLATE -1
typedef struct
{
	long target;
	long tolerance;
	unsigned char* pwm;
	long int_error;
	long prev_error;
  int kp_num, kp_den, ki_num, ki_den, kd_num, kd_den;
  int int_err_max;
  int p_out_max, i_out_max;
  int output_min, output_max;
} PIDData;

/* Initialize this module. Must call this during init! */
void Motion_Init(void);

void Init_PIDData_Struct(PIDData* control, unsigned char* pwm);
void Set_PIDData_Target(PIDData* control, long new_target);
unsigned char PID_Control(PIDData* control, long current);
void Init_Drive_Straight(long ticks);
void Init_Drive_Straight_Limited(long ticks, int max_power);
// MUST set target number of ticks using Init_Drive_Straight
unsigned char Drive_Ticks(int heading, int integral_reset);
// Negative for right, positive for left
void Init_Drive_Turn(long ticks);
// MUST set target number of ticks externally using Init_Drive_Turn()
unsigned char Turn_Ticks(void);


//prototype functions


//Drive_Angle(Gyro Angle to drive to, max PWM delta allowed, 
//            Reset Integral Error, Forward PWM value)
//            FWDSPEED = 0 means make left turn, 
//            FWDSPEED < 0 means make right turn    
void Drive_Angle(int HEADING, int max_turn, int integral_reset, int fwdspeed);
int Turn_180(int lanenumber);

// jim's functions, don't touch!
void reset_adjust_rates(char right_bit, char left_bit);
void Check_Left_Adjust_rate(int Left_Target_PWM);
void Check_Right_Adjust_rate(int Right_Target_PWM);
void Angle_Rate_Turn_SW(int Left_Target_PWM, int Right_Target_PWM, int rate_Angle);
void Angle_Rate_Turn_TW(int Left_Target_PWM, int Right_Target_PWM, int rate_Angle);

void Change_Lane(int Target_Lane);
void return_to_Orgin_Heading(void);
void passes_216_in(void);
void passes_108_in(void);
void Turn_15_Degree_Right(void);
void Turn_15_Degree_Left(void);
int Get_Task_Done(void);

#endif // _motion_h

