#include <cc3_math.h>
#include <stdbool.h>
#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <cc3.h>
#include <cc3_ilp.h>
#include <cc3_img_writer.h>
#include <cc3_jpg.h>
#include <cc3_color_track.h>
#include <cc3_color_info.h>
#include <cc3_frame_diff.h>
#include "polly.h"
#include "packets.h"

extern bool raw_mode_output;
extern bool raw_mode_no_confirmations;
extern bool raw_mode_input;
extern uint8_t t_pkt_mask;
void cmucam2_write_t_packet (cc3_track_pkt_t *pkt)
{

  // cap at 255
  if (pkt->centroid_x > 255)
    pkt->centroid_x = 255;
  if (pkt->centroid_y > 255)
    pkt->centroid_y = 255;
  if (pkt->x0 > 255)
    pkt->x0 = 255;
  if (pkt->x1 > 255)
    pkt->x1 = 255;
  if (pkt->y1 > 255)
    pkt->y1 = 255;
  if (pkt->y0 > 255)
    pkt->y0 = 255;
  if (pkt->num_pixels > 255)
    pkt->num_pixels = 255;
  if (pkt->int_density > 255)
    pkt->int_density = 255;

  // values to print
  uint8_t p[8];

  if (pkt->num_pixels == 0) 
  {
    p[0] = p[1] = p[2] = p[3] = p[4] = p[5] = p[6] = p[7] = 0;
  }
  else 
  {
    p[0] = pkt->centroid_x;
    p[1] = pkt->centroid_y;
    p[2] = pkt->x0;
    p[3] = pkt->y0;
    p[4] = pkt->x1;
    p[5] = pkt->y1;
    p[6] = pkt->num_pixels;
    p[7] = pkt->int_density;
  }

  uint8_t mask = t_pkt_mask;
  if (raw_mode_output) 
  {
    putchar (255);
  }
  printf ("T");

  // print out fields using mask
  for (int i = 0; i < 8; i++) 
  {
    if (mask & 0x1) 
    {
      if (raw_mode_output) 
      {
        raw_print (p[i]);
      }
      else 
      {
        printf (" %d", p[i]);
      }
    }
    mask >>= 1;
  }


  print_cr ();
}

void cmucam2_write_h_packet (cc3_histogram_pkt_t *pkt)
{
  uint32_t i;
  uint32_t total_pix;

  total_pix = cc3_g_pixbuf_frame.width * cc3_g_pixbuf_frame.height;

  if (raw_mode_output) {
    putchar (255);
  }
  printf ("H");

  for (i = 0; i < pkt->bins; i++) {
    pkt->hist[i] = (pkt->hist[i] * 256) / total_pix;
    if (pkt->hist[i] > 255)
      pkt->hist[i] = 255;

    if (raw_mode_output) {
      raw_print (pkt->hist[i]);
    }
    else {
      printf (" %d", pkt->hist[i]);
    }
  }

  print_cr ();
}

void print_ACK ()
{
  if (!raw_mode_no_confirmations)
    printf ("ACK\r");
}

void print_NCK ()
{
  if (!raw_mode_no_confirmations)
    printf ("NCK\r");
}

void print_prompt ()
{
  printf (":");
}

void print_cr ()
{
  if (!raw_mode_output) {
    printf ("\r");
  }
}
void raw_print (uint8_t val)
{
  if (val == 255) 
  {
    putchar (254);              // avoid confusion
  }
  else 
  {
    putchar (val);
  }
}