/*******************************************************************************
*
*	TITLE:		control.c 
*
*	VERSION:	0.1                           
*
*	DATE:		01-29-2008
*
*	AUTHOR:		Team 1511
*
*	DESCRIPTION:	This file contains functions for controlling the arm and manipulator.
*******************************************************************************/

#ifndef _control_h
#define _control_h

// Minimum movement of joystick required to actuate arm
#define ARM_CONTROL_DEADBAND 30

// Lowest arm POT value
//#define ARM_POT_MIN 54
#define ARM_POT_MIN 244

// Max arm POT value
//#define ARM_POT_MAX (ARM_POT_MIN + 476)
#define ARM_POT_MAX (ARM_POT_MIN + 268)

// Threshold for comparison against min/max for arm all the way
// up/down
#define ARM_POT_THRESHOLD 100



// PWM delta used when driving wrist if POT is broken
#define WRIST_MANUAL_SPEED 80

// POT value to step by with button presses up and down (for PID)
#define WRIST_TARGET_STEP 7

// Lowest POT value for the wrist while the arm is up
//#define WRIST_POT_MIN 162 -- was 538
// minus 10
#define WRIST_POT_MIN 260

// Lowest POT value for the wrist while the arm is down 
//also use for AUTOARM_TARGET_PICK
//#define WRIST_POT_MIN_FLOOR 372
//#define WRIST_POT_MIN_FLOOR 816
#define WRIST_POT_MIN_FLOOR (WRIST_POT_MIN + 293)  //was 723

// Wrist POT value for preparation of hurdle
#define WRIST_POT_HURDLE (WRIST_POT_MIN + 463)

// Wrist POT value for completion of hurdle
#define WRIST_POT_HURDLE_FINISH (WRIST_POT_MIN + 204) 

// Wrist POT value at which to open the gripper when finishing
// a hurdle.
#define WRIST_POT_HURDLE_OPEN (WRIST_POT_MIN + 294)

// Wrist POT value for pickup
//#define WRIST_POT_PICKUP (WRIST_POT_MIN_FLOOR + 65)
#define WRIST_POT_PICKUP WRIST_POT_MIN_FLOOR

// Greatest POT value for the wrist
//1450  -- now 1302 in park, so 
#define WRIST_POT_MAX (WRIST_POT_MIN + 778)

// Greatest POT value with a ball and arm in the air
#define WRIST_POT_MAX_HOLD (WRIST_POT_MIN + 330)

// POT value at which wrist must be at or above in order to
// stay within the 80 inch cylinder for arm raise/lower movement
#define WRIST_POT_MIN_RAISE (WRIST_POT_MIN + 550) 
#define WRIST_POT_MIN_LOWER (WRIST_POT_MIN + 550) 

// Wrist angle to park robot within starting box
#define WRIST_POT_PARK WRIST_POT_MAX



// Number of 26.2ms ticks to force pump on for after manipulator
// cylinder is fired.  This should be the amount of time it takes 
// to replace the air used to actuate the manipulator cylinder.
#define MANIP_PUMP_TICKS 38

// Number of 26.2ms ticks to force pump on for after arm cylinder
// is fired.  Note that pump will stay on for quite a while after 
// this because of the pressure switch; this merely starts the 
// pump up before the pressure switch goes.
#define ARM_PUMP_TICKS 38



void Initialize_Arm(void);

#define AUTOARM_TARGET_PARK 1
#define AUTOARM_TARGET_PICKUP 2
#define AUTOARM_TARGET_HURDLE 3
#define AUTOARM_TARGET_HITBALL 4
#define AUTOARM_TARGET_PICK 5
#define AUTOARM_TARGET_HURDLE_FINISH 6
int AutoArm(int target);
void Arm_Control(void);
unsigned char Pump_Precharge(void);

#endif
