#include <stdio.h>
#include "iomap.h"
#include "user_routines.h"
#include "gyro.h"
#include "encoder.h"
#include "hybrid.h"
#include "motion.h"
#include "control.h"

static void Drive_Front_Back (int reset);
static void Drive_Fast_Common (int pokeyCommands, int lanechange_distance, int origstartpos);

static int state = -3, slowdown=0, lastIR = 0;
static int defense_state = 0;
static int drivelane_state = 0;
static long leftEnc = 0, rightEnc=0;
static int heading = 0;
static int stop = 1;
static char return_state = 0;
static int pokestate = 0;
static int drive_turn_finished = 0;
int lane_change = 0; //flag set if we are lane changing to make drive_ticks turn slow

//speed for hybrid_drive (IR command)
static unsigned char hybrid_speed = 127;


void Hybrid_Reset(void)
{
        state = -3;
        defense_state = 0;
        pokestate = 0;
        slowdown = 0;
        lastIR = 0;
        leftEnc = 0;
        rightEnc = 0;
        heading = 0;
        stop = 1;
        return_state = 0;
        hybrid_speed = 127;
        drivelane_state = 0;
        drive_turn_finished = 0;
}



//53" between lanes, 324" to drive out, 9.3 degrees per lane.

//The idea for lane change is to have 2 additional states 
//  for driving out that get used if you need to lane change.
//  Those states will force the 9.3 or 18.6degree heading for a 
//  Drive_Ticks() distance of 328" or 341". (1 or 2 lanes)
//  Then, the original 2 states for driving out will have their
//  distance changed to FINISH_LINE_TO_END, but heading stays the same (0 or 180)
//  Also, autoposition needs to be updated so that our arc'd turn is correct

//Button 1 - Lane 1 (inside lane)
//Button 2 - Lane 3 (outside lane)


static void Drive_Fast_Common (int pokeyCommands, int lanedistance, int origstartpos) //pokeycommands = 1 if in poke mode
{
        int IR = getIR(0);
        int curAngle = Get_Gyro_Angle();    //use this once instead of calling Get_Gyro_Angle() over and over
        
        char return_state = 0;

        if (!pokeyCommands && IR == 2)    //stop and spin right 5 degrees
        {
                if (lastIR != IR)   //first time into this command, set target angle
                {
                    heading = curAngle + 50;
                    //save encoder values so we can restore on exit
                    leftEnc = Get_Left_Encoder_Count();
                    rightEnc = Get_Right_Encoder_Count();
                    lastIR = IR;
                }
    
                Drive_Angle(heading, 30, 1, -1); //turn right 5 degrees

                if(curAngle >= heading)  //finished turning, may never complete if 
                {
                    getIR(1);   //reset our IR command so we stop executing the above if
                    printf("Done turning 5 degrees right\r\n");

                    //now that we're done turning, restore encoder values
                    Set_Left_Encoder_Count(leftEnc);
                    Set_Right_Encoder_Count(rightEnc);
            	}
                else
                    return; //don't process the state machine while turning right

        }

printf("STATE: %d %d\r\n", (int)state, (int)heading);
        
        switch (state)
        {
            case -3: 
                digout_coast = 0;
                // Set up a lane change if requested, skip to state 0 otherwise
                if (lanedistance == 0)
                {
                    state = 0;
                }
                else
                {
                    lane_change = 1;
                    Init_Drive_Straight((long)lanedistance);
                    Drive_Ticks(heading, 1);
                    state++;
                }
                break;
            case -2:
                // Lane change.
                state = Drive_Ticks(heading, 0) + (int)-2;
                break;
            case -1:
                // Now go straight ahead again after the lane change.
                heading = 0;
                // 220 inches is amount of forward movement we've gone
                // when we do a lane change, based on the heading angle
                // and lanedistance provided by Hybrid_Pokey.
                lanedistance = 220 * TICKS_PER_INCH;
                state = 0;
                break;
            case 0:
                lane_change = 0;
                if (origstartpos == 3) {  //1930 - 1100
                  Init_Drive_Straight ((long)START_TO_LANE_END_OUTSIDE - (long)lanedistance);
                } else {
                  if (autoposition == 1)
                    Init_Drive_Straight ((long)START_TO_LANE_END - (long)lanedistance + 4L*TICKS_PER_FOOT);
                  else
                    Init_Drive_Straight ((long)START_TO_LANE_END - (long)lanedistance);
                }
                Drive_Ticks(heading, 1);  //resets Drive_ticks            
                state = 1;
                break;

            //allow robocoach to override Drive_Ticks and turn a bit earlier.
            case 1:
                if(!pokeyCommands && IR == 1)	//turn 90 left now! skip to state 60
                {
                    state = 60;
                    return_state = 5;
                }
                else
                    state = Drive_Ticks(heading, 0) + 1;
                break;
            case 2:
               // printf("Left Enc: %d\r\n", (int)Get_Left_Encoder_Count());
                Init_Drive_Straight (SLOW_DOWN_LENGTH);
                //Drive_Ticks(heading, 1);
                state = 3;  //resets Drive_ticks            
                break;
            case 3:
                digout_coast = 1;
                if(!pokeyCommands && IR == 1)	//turn 90 left now! skip to state 60
                {
                    state = 60;
                    return_state = 6;
                }
                else
                    state = Drive_Ticks(heading, 0) + 3;
                if (state == 4)
                    {
                      Reset_Right_Encoder_Count();  
                      digout_coast = 1;
                    }
                break;
            case 4:
                state = Turn_180(autoposition) + 4;
                drive_turn_finished = 1;
                break;
            case 5:
                digout_coast = 0;
                if (autoposition == 3)
                  Init_Drive_Straight (LANE_DIV_LENGTH - (60 * TICKS_PER_INCH));
                else
                  Init_Drive_Straight (LANE_DIV_LENGTH);
                state = 6;
                heading = -1800;
                Drive_Ticks(heading, 1);
                //drive_turn_finished = 1;
                break;
             case 6:
                if(!pokeyCommands && IR == 1)	//turn 90 left now! skip to state 60
                {
                    state = 60;
                    return_state = 11;
                }
                else
                    state = Drive_Ticks(heading, 0) + 6;
                break;
            case 7:
                Init_Drive_Straight (SLOW_DOWN_LENGTH);
                state = Drive_Ticks(heading, 1) + 8;  //resets Drive_ticks        
                Reset_Right_Encoder_Count(); 
                break;    
            case 8: 
               if (!pokeyCommands && IR == 1)
               {
                    state = 60;
                    return_state = 11;
               }
               else
                    state = Drive_Ticks (heading, 0) + 8;
               break;
            case 9:
                digout_coast = 1;
                Reset_Gyro_Angle();
                Turn_180(0);
                Reset_Right_Encoder_Count();
                state = 10;
                break;
            case 10:
                state = Turn_180(autoposition) + 10;
                break;
            case 11:
                Init_Drive_Straight (LANE_DIV_LENGTH);
                state = 12;
                heading = -1800;
                Drive_Ticks(heading, 1);
                break;
            case 12:
                state = Drive_Ticks(heading, 0) + 12;
                break;
            case 13:
                pwm_left_drive = 127;
                pwm_right_drive = 127;
                break;
                
            //interrupt and turn 90 left cases
            case 60:
                digout_coast = 0;
                Drive_Angle(-900, 10,1,0);
                if(curAngle < -850)
                    state = 61;
                break;
            case 61:
                Init_Drive_Straight(FINISH_LINE_TO_END);
                state = 62;
                heading = -900;
                Drive_Ticks(heading,1);
                break;
            case 62:
                state = Drive_Ticks(heading,0) + 62;
                if (state == 63)
                	getIR(1);	//reset IR so we can see when another cmd comes through
                break;
            case 63:
                pwm_left_drive = 127;
                pwm_right_drive = 127;
                if(IR == 1)		//turn 90 again!
                    state = 64;
                break;
            case 64:
                Drive_Angle(-1800,10,1,0);
                if(curAngle < -1750)
                {
                    digout_coast = 1;
                    state = return_state;	//we're done with manual "Turn_180", drive straight!
                    getIR(1);   //reset our IR command
            	}
                break;   
    }

                if (!pokeyCommands && IR == 3)
                {
                //Alter speed/pause/stop       

                //every 'click' will increase the slowdown amount more.
                //every loop without a 'click' will decrease the slowdown amount  
                    slowdown+=6;    
                    if (slowdown >= 127)
                        slowdown = 127;
                    getIR(1);   //reset             
                }
                else if (IR == 4)
                {
                    pwm_left_drive = 127;
                    pwm_right_drive = 127;
 //                   getIR(1);   //reset
                }
                else   
                {
                    //if we're not being commanded to slow down, speed back up
                    slowdown-=2; 
                    if (slowdown < 0)
                        slowdown=0;
                }

                //modify drive speeds by our 'slowdown' commmand
                pwm_left_drive-=slowdown;
                pwm_right_drive-=slowdown;
                
                //don't go backward!
                if (pwm_left_drive < 127)
                    pwm_left_drive = 127;
                if (pwm_right_drive < 127)
                    pwm_right_drive = 127;   
                    
                //save our last IR command
                lastIR = IR;
}

void Hybrid_Drive_Fast(void)
{
    Drive_Fast_Common(0, 0, autoposition);
}


void Hybrid_Pokey(int far_mode)
{
    static int lanedistance = 0;
    static int origstartpos = 1;
    int IR = getIR(0);

    // Modify lane position, initial heading, and drive distances
    // based on ball location derived from starting lane and IR
    if (pokestate == 0) //first time through
    {
      int ballpos, diff;

      //Button 1 == lane 1 (inside)
      //Button 2 == lane 3 (outside)
      if (IR == 1)
          ballpos = 1;
      else if (IR == 2)
          ballpos = 3;
      else
          ballpos = 2;

      diff = ballpos - autoposition;

      origstartpos = autoposition;
      autoposition = ballpos;
      
      // CAUTION: These values derived from 53 inch lane separation
      //          and leaving about 6 feet to drive straight into the ball.
      //          If you change these, you will want to change the value
      //          used for distance to drive forward in Drive_Fast_Common()
      if (diff == 1) {
          heading = (int)diff * (int)135;
          lanedistance = (int)184 * TICKS_PER_INCH;
      } else if (diff == -1) {
          heading = (int)diff * (int)135;
          lanedistance = (int)114 * TICKS_PER_INCH;
      } else if (diff == 2) {
          heading = (int)257 * ((int)diff / 2);
          lanedistance = (int)244 * TICKS_PER_INCH;
      } else if (diff == -2) {
          heading = (int)257 * ((int)diff / 2);
          lanedistance = (int)200 * TICKS_PER_INCH;
      }

      pokestate = AUTOARM_TARGET_HITBALL;
    }

    if (IR == 3 || (!far_mode && drive_turn_finished))
        pokestate = AUTOARM_TARGET_PARK;

    AutoArm(pokestate);
    Drive_Fast_Common(1, lanedistance, origstartpos);
}


void Hybrid_Defense (void)
{
        int IR = getIR(0);

        //initially this will drive out 7 ft away from wall before awaiting commands
        switch (defense_state)
        {
            case 0: 
                Init_Drive_Straight ((84 * TICKS_PER_INCH) - (4 * TICKS_PER_INCH));
                state = Drive_Ticks(0, 1) + 1;
                break;
            case 1:
                state = Drive_Ticks(0, 0) + 1;
                break;
            case 2:
                pwm_left_drive = 127;
                pwm_right_drive = 127;
                state = 3;
                break;
            case 3:
                if (IR == 4) //STOP!!!
                {
                    pwm_left_drive = 127;
                    pwm_right_drive = 127;
                }
                else if (IR == 1)   //drive back and forth
                {
                    if (IR != lastIR)
                        Drive_Front_Back (1);                      
                    else
                        Drive_Front_Back(0);
                }
                else if (IR == 2)   //spin in place
                {
                    pwm_left_drive = 127 + 40;
                    pwm_right_drive = 127 - 40;
                }
               else if (IR == 3)    //drive in a circle
                {
                    pwm_left_drive = 127 + 25;
                    pwm_right_drive = 127 + 40;
                }
                break;
            
      }
            //save what our last command was so we can check for change
            lastIR = IR;
    //printf("State %d pwmright %d pwmleft %d\r\n", state, pwm_right_drive, pwm_left_drive);
}


static void Drive_Front_Back (int reset)
  {        
          static int state = 0;
          if (reset)
              state = 0;

          switch (state)
          {            
            case 0:
              Init_Drive_Straight ((84 * TICKS_PER_INCH) - (4 * TICKS_PER_INCH));
              state = Drive_Ticks(0,1) + 1;
              break;
            case 1:
              state = Drive_Ticks(0,0) + 1;
              break;
            case 2:
              Init_Drive_Straight ((-162 * TICKS_PER_INCH) - (-4 * TICKS_PER_INCH));
              state = Drive_Ticks (0,1) + 3;
              break;
            case 3:
              state = Drive_Ticks(0,0) + 3;
              break;
            case 4:
              Init_Drive_Straight ((84 * TICKS_PER_INCH) - (4 * TICKS_PER_INCH));
              state = Drive_Ticks (0,1) + 5;
              break;
            case 5:
              state = Drive_Ticks(0,0) + 5;
              break; 
            case 6:
              state = 2;
              break;
          }
    
}


void Hybrid_Robocoach_drive (void)
{
	int IR;
	IR = getIR(0);

  //Set to brake mode
  digout_coast = 0;

  if (IR == 4 || (IR == 0 && stop))
	{
    heading = 0;
    Drive_Angle(heading, 30, 1, 200);
    Reset_Gyro_Angle();
		pwm_left_drive = 127;
		pwm_right_drive = 127; 
    stop = 1;
    hybrid_speed = 165;
	} 
  else
  {
    if (IR == 3)
    {
      // Straight
      Reset_Gyro_Angle();
      heading = 0;
      hybrid_speed = 200;
      getIR(1);
      stop = 0;
    }
  	else if (IR == 1)
  	{
      // Left
      heading -= 50;
      hybrid_speed = 165;
      getIR(1);
      stop = 0;
  	}
  	else if (IR == 2)
  	{
      // Right
      heading += 50;
      hybrid_speed = 165;
      getIR(1);
      stop = 0;
  	}

    Drive_Angle(heading, 30, 0, hybrid_speed);

  }

  return;
}

void Hybrid_Drive_Lane(int irenabled)
{
    if (pokestate == 0)
        // First time called - set arm tgt to park
        pokestate = AUTOARM_TARGET_PARK;

    if (irenabled) {
        int IR = getIR(0);
        if (IR == 1)
            pokestate = AUTOARM_TARGET_HITBALL;
        else if (IR == 4)
            // Stop
            drivelane_state = 2;
        else if (IR == 3) {
            drivelane_state = 0;
            getIR(1);
        } else
            pokestate = AUTOARM_TARGET_PARK;
            
        AutoArm(pokestate);
    }

    switch (drivelane_state)
    {
        case 0:
            Init_Drive_Straight((long)START_TO_LANE_END_OUTSIDE);
            Drive_Ticks(0,1);
            ++drivelane_state;
            break;
        case 1:
            drivelane_state = 1 + Drive_Ticks(0, 0);
            break;
        default:
            pwm_left_drive = pwm_right_drive = 127;
            break;
    }
}




