/*
 * This file maps all the IFI I/O port names
 * to Team 1511 robot input/output functional
 * names. This makes it easy to handle pin reallocations
 * and remember variable names.
 *
 * These should be kept in sync with the rc and oi Excel
 * spreadsheet mappings of the RC and OI pinouts. Changes
 * need to be coordinated with Electrical.
 */

#ifndef IOMAP_H
#define IOMAP_H

#include "ifi_default.h"
#include "ifi_aliases.h"


/*
 * PWM Aliases.
 * 5 - 16 are currently unused
 */
#define pwm_left_drive                 pwm01
#define pwm_right_drive                pwm02
#define pwm_wrist                      pwm03
#define pwm_ir_servo                   pwm04

/*
 * Relay Aliases
 * 5 - 8 Unused
 */
#define relay_arm_up                   relay1_fwd
#define relay_arm_down                 relay1_rev
#define relay_gripper_close            relay2_fwd
#define relay_gripper_open             relay2_rev
#define relay_poker_up                 relay3_fwd
#define relay_poker_down               relay3_rev
#define relay_pump                     relay4_fwd

/* 
 * Analog inputs
 * 7 - 16 currently unused
 */
extern unsigned int anain_gyro;
#define anain_gyro_in                  1
extern unsigned int anain_wrist_pot;
#define anain_wrist_pot_in             2
extern unsigned int anain_shoulder_pot;
#define anain_shoulder_pot_in          3


/* Digital I/O
 * NOTE: If you change the direction of the pin, you
 *       must change the init code in 
 *       user_routines.c::User_Initialization()
 * 1, 2, 5, 17-18 are not in use
 */

// For the encoders, enable the correct one, and also 
//   alias the functions
#define digin_left_wheel_enc           rc_dig_in03
#define ENABLE_ENCODER_3
#define ENCODER_3_TICK_DELTA           1
#define Get_Left_Encoder_Count         Get_Encoder_3_Count
#define Set_Left_Encoder_Count         Set_Encoder_3_Count
#define Reset_Left_Encoder_Count       Reset_Encoder_3_Count


#define digin_right_wheel_enc          rc_dig_in04
#define ENABLE_ENCODER_4
#define ENCODER_4_TICK_DELTA	       -1
#define Get_Right_Encoder_Count        Get_Encoder_4_Count
#define Set_Right_Encoder_Count        Set_Encoder_4_Count
#define Reset_Right_Encoder_Count      Reset_Encoder_4_Count

#define digin_gripper_open             rc_dig_in06

// Encoder phase B inputs
#define digin_left_wheel_enc_phaseb    rc_dig_in07
#define ENCODER_3_PHASE_B_PIN          rc_dig_in07
#define digin_right_wheel_enc_phaseb   rc_dig_in08
#define ENCODER_4_PHASE_B_PIN          rc_dig_in08

// back IR board inputs
#define digin_ir1                      rc_dig_in10
#define digin_ir2                      rc_dig_in12
#define digin_ir3                      rc_dig_in09
#define digin_ir4                      rc_dig_in11

//front IR board inputs
#define digin_2ir1                     rc_dig_in14
#define digin_2ir2                     rc_dig_in16
#define digin_2ir3                     rc_dig_in13
#define digin_2ir4                     rc_dig_in15

#define digin_pressure_switch          rc_dig_in17

#define digout_coast                   rc_dig_out18

/* 
 *OI Inputs
 */

#define oi_right_drive                 p2_x
#define oi_left_drive                  p2_y
#define oi_slow_right                  p2_sw_trig
#define oi_slow_left                   p2_sw_top
#define oi_turbo_right                 p2_sw_aux1
#define oi_turbo_left                  p2_sw_aux2

#define oi_arm                         p4_x
#define oi_gripper_open                p4_sw_trig
#define oi_gripper_close               p4_sw_top
#define oi_armpos_park                 p4_sw_aux1
#define oi_armpos_overpass             p4_sw_aux2

/* These are mapped in chicklet_analog_decode */
extern unsigned char oi_armpos_pickup;
extern unsigned char oi_armpos_hitball;
extern unsigned char oi_armpos_hurdle;
extern unsigned char oi_armpos_hurdlefinish;
extern unsigned char oi_wrist_up;
extern unsigned char oi_wrist_down;


#define oi_debug_drive                 p1_sw_trig
#define oi_debug_button2               p1_sw_top
#define oi_debug_button3               p1_sw_aux1
#define oi_debug_switch                p1_sw_aux2

#define oi_brake                       p3_sw_trig
#define oi_switch2                     p3_sw_top
#define oi_broken_shoulder             p3_sw_aux1
#define oi_broken_wrist                p3_sw_aux2

#define oi_auto_ones_b0                p3_x
#define oi_auto_ones_b1                p3_y
#define oi_auto_ones_b2                p3_wheel
#define oi_auto_tens_b0                p1_x
#define oi_auto_tens_b1                p1_y
#define oi_auto_tens_b2                p1_wheel

#define oi_starting_pos                p3_aux
#define oi_auto_delay                  p1_aux

#define oi_led_debug                   Pwm1_green//Relay1_green
#define oi_led_ircmd1                  Relay1_green
#define oi_led_ircmd2                  Relay2_green//Pwm1_red
#define oi_led_ircmd3                  Relay1_red//Pwm2_green
#define oi_led_ircmd4                  Relay2_red//Pwm2_red
#define oi_led_gripper_closed          Pwm1_red//Relay1_red
#define oi_led_arm_pot_bad             Pwm2_green//Relay2_green
#define oi_led_wrist_pot_bad           Pwm2_red//Relay2_red

#endif

